'use strict';

var css = require('@master/css');
var vscodeLanguageserverProtocol = require('vscode-languageserver-protocol');
var sortCompletionItems = require('./sort-completion-items.cjs');
var getCssDataDocumentation = require('./get-css-data-documentation.cjs');
var cssDataProvider = require('./css-data-provider.cjs');

function getQueryCompletionItems(css$1 = new css.MasterCSS(), triggerCharacter = css.AT_SIGN, syntax) {
    const atDataList = cssDataProvider.provideAtDirectives();
    const completionItems = [];
    if (!css.QUERY_COMPARISON_OPERATORS.includes(triggerCharacter)) {
        [
            'media',
            'container',
            'supports'
        ].forEach((name)=>{
            completionItems.push({
                label: triggerCharacter + name + '()',
                filterText: name + '()',
                insertText: name + '()',
                sortText: name + '()',
                documentation: getCssDataDocumentation.getCSSDataDocumentation(atDataList.find(({ name })=>name === '@' + name), {
                    generatedCSS: css.generateCSS([
                        syntax + name + '()'
                    ], css$1),
                    docs: '/guide/at'
                })
            });
        });
    }
    const handleNumberNodes = (atRule, token)=>{
        const numberNode = css.getSingleAtNumberRuleNode(atRule.nodes);
        if (numberNode) {
            const prevComparisonCharacter = syntax.charAt(syntax.length - 2);
            const comparisonCharacter = prevComparisonCharacter === '>' || prevComparisonCharacter === '<' ? prevComparisonCharacter + triggerCharacter : triggerCharacter;
            const text = css.generateAt(css.parseAt(comparisonCharacter + token, css$1));
            completionItems.push({
                label: comparisonCharacter + token,
                sortText: vscodeLanguageserverProtocol.CompletionItemKind.Keyword + comparisonCharacter + String(Math.round(numberNode.value)).padStart(12, '0'),
                filterText: token,
                insertText: token,
                detail: text,
                kind: vscodeLanguageserverProtocol.CompletionItemKind.Keyword,
                documentation: getCssDataDocumentation.getCSSDataDocumentation(undefined, {
                    generatedCSS: css.generateCSS([
                        syntax + token
                    ], css$1),
                    docs: '/guide/at'
                })
            });
            return true;
        }
    };
    css$1.atRules.forEach((atRule, token)=>{
        const completionItem = {
            filterText: token,
            insertText: token,
            documentation: getCssDataDocumentation.getCSSDataDocumentation(undefined, {
                generatedCSS: css.generateCSS([
                    syntax + token
                ], css$1),
                docs: '/guide/at'
            })
        };
        if ([
            css.AT_SIGN,
            ...css.QUERY_LOGICAL_OPERATORS
        ].includes(triggerCharacter)) {
            if (handleNumberNodes(atRule, token)) return;
            const text = css.generateAt(css.parseAt(triggerCharacter + token, css$1));
            completionItems.push({
                ...completionItem,
                label: triggerCharacter + token,
                detail: text,
                sortText: vscodeLanguageserverProtocol.CompletionItemKind.Keyword + token,
                kind: vscodeLanguageserverProtocol.CompletionItemKind.Keyword,
                documentation: getCssDataDocumentation.getCSSDataDocumentation(undefined, {
                    generatedCSS: css.generateCSS([
                        syntax + token
                    ], css$1),
                    docs: '/guide/at'
                })
            });
        } else if (css.QUERY_COMPARISON_OPERATORS.includes(triggerCharacter)) {
            handleNumberNodes(atRule, token);
        }
    });
    return sortCompletionItems(completionItems);
}

module.exports = getQueryCompletionItems;
